@@ -26,6 +26,8 @@
 #
 ########################################################################
 
+=encoding UTF-8
+
 =head1 NAME
 
 Locale::Po4a::Dia - Convert uncompressed Dia diagrams from/to PO files
@@ -91,14 +93,14 @@ sub initialize {
 	my %options = @_;
 
 	$self->SUPER::initialize(%options);
-	$self->{options}{'tagsonly'}=1;
 	$self->{options}{'nostrip'}=1;
-	$self->{options}{'tags'}.=' <dia:string>';
+	$self->{options}{'_default_translated'}.=' <dia:string>';
 	$self->treat_options;
 }
 
 sub found_string {
 	my ($self,$text,$ref,$options)=@_;
+	return $text if $text =~ m/^\s*$/s;
 
 	#We skip the paper type string
 	if ( $self->get_path() !~ /<dia:diagramdata>/ ) {
