@@ -28,6 +28,8 @@
 #
 ########################################################################
 
+=encoding UTF-8
+
 =head1 NAME
 
 Locale::Po4a::Docbook - Convert Docbook XML documents from/to PO files
@@ -89,7 +91,6 @@ sub initialize {
 	my %options = @_;
 
 	$self->SUPER::initialize(%options);
-	$self->{options}{'tagsonly'}=1;
 	$self->{options}{'wrap'}=1;
 	$self->{options}{'doctype'}=$self->{options}{'doctype'} || 'docbook xml';
 
@@ -213,7 +214,7 @@ sub initialize {
 	# NOTE: the attributes might be translated
 	$self->{options}{'_default_translated'} .= " <audiodata>";
 	$self->{options}{'_default_placeholder'} .= " <audiodata>";
-	$self->{options}{'attributes'}.=' <audiodata>fileref';
+	$self->{options}{'_default_attributes'}.=' <audiodata>fileref';
 
 	# audioobject; does not contain text;
 	# NOTE: might be contaioned in a inlinemediaobject
@@ -766,7 +767,7 @@ sub initialize {
 	# v4, not in v5
 	$self->{options}{'_default_untranslated'} .= " <graphic>";
 	$self->{options}{'_default_inline'} .= " <graphic>";
-	$self->{options}{'attributes'}.=' <graphic>fileref';
+	$self->{options}{'_default_attributes'}.=' <graphic>fileref';
 
 	# graphicco; does not contain text; Formatted as a displayed block.
 	# v4, not in v5
@@ -862,7 +863,7 @@ sub initialize {
 	# as a displayed block, depending on context
 	$self->{options}{'_default_translated'} .= " <imagedata>";
 	$self->{options}{'_default_inline'} .= " <imagedata>";
-	$self->{options}{'attributes'}.=' <imagedata>fileref';
+	$self->{options}{'_default_attributes'}.=' <imagedata>fileref';
 
 	# imageobject; does not contain text; May be formatted inline or
 	# as a displayed block, depending on context
@@ -1860,7 +1861,7 @@ sub initialize {
 	# NOTE: could be in the inline class
 	$self->{options}{'_default_untranslated'} .= " <textdata>";
 	$self->{options}{'_default_break'} .= " <textdata>";
-	$self->{options}{'attributes'}.=' <textdata>fileref';
+	$self->{options}{'_default_attributes'}.=' <textdata>fileref';
 
 	# textobject; does not contain text; Formatted inline or as a displayed block
 	# NOTE: could be in the inline class
@@ -1995,7 +1996,7 @@ sub initialize {
 	# videodata; contains text; Formatted inline or as a displayed block
 	$self->{options}{'_default_untranslated'} .= " <videodata>";
 	$self->{options}{'_default_break'} .= " <videodata>";
-	$self->{options}{'attributes'}.=' <videodata>fileref';
+	$self->{options}{'_default_attributes'}.=' <videodata>fileref';
 
 	# videoobject; contains text; Formatted inline or as a displayed block
 	$self->{options}{'_default_untranslated'} .= " <videoobject>";
@@ -2033,7 +2034,7 @@ sub initialize {
 
 # ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZ
 
-	$self->{options}{'attributes'}.='
+	$self->{options}{'_default_attributes'}.='
 		lang
 		xml:lang';
 
