@@ -21,10 +21,13 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #
 ########################################################################
 
+=encoding UTF-8
+
 =head1 NAME
 
 Locale::Po4a::Guide - Convert Guide XML documents from/to PO files
@@ -83,8 +86,7 @@ sub initialize {
 
 #TODO: <include href="..."> includes a file
 	$self->SUPER::initialize(%options);
-	$self->{options}{'tagsonly'}=1;
-	$self->{options}{'tags'}.='
+	$self->{options}{'_default_translated'}.='
 		w<abstract>
 		<author>
 		<b>
@@ -119,7 +121,7 @@ sub initialize {
 		<var>
 		<version>
 		w<warn>';
-	$self->{options}{'attributes'}.='
+	$self->{options}{'_default_attributes'}.='
 		<author>title
 		<figure>caption
 		<figure>link
@@ -128,7 +130,7 @@ sub initialize {
 		<guide>link
 		<p>by
 		<pre>caption';
-	$self->{options}{'inline'}.='
+	$self->{options}{'_default_inline'}.='
 		<b>
 		<brite>
 		<c>
