@@ -1,5 +1,5 @@
 # Locale::Po4a::Po -- manipulation of po files
-# $Id: Po.pm,v 1.92 2008-07-14 19:42:01 nekral-guest Exp $
+# $Id: Po.pm,v 1.97 2009-12-30 21:12:31 nekral-guest Exp $
 #
 # This program is free software; you may redistribute it and/or modify it
 # under the terms of GPL (see COPYING).
@@ -8,6 +8,8 @@
 # Modules and declarations
 ############################################################################
 
+=encoding UTF-8
+
 =head1 NAME
 
 Locale::Po4a::Po - po file manipulation module
@@ -85,6 +87,7 @@ use strict;
 use warnings;
 
 use Carp qw(croak);
+use File::Basename;
 use File::Path; # mkdir before write
 use File::Copy; # move
 use POSIX qw(strftime floor);
@@ -218,6 +221,10 @@ sub read {
                           dgettext("po4a",
                                    "Please provide a non-null filename"));
 
+    my $lang = basename($filename);
+    $lang =~ s/\.po$//;
+    $self->{lang} = $lang;
+
     my $fh;
     if ($filename eq '-') {
         $fh=*STDIN;
@@ -408,7 +415,7 @@ sub write{
                 $output .= format_comment($comment, ". ")
             }
         }
-        $output .= format_comment($self->{po}{$msgid}{'type'}," type: ")
+        $output .= format_comment($self->{po}{$msgid}{'type'},". type: ")
             if    defined($self->{po}{$msgid}{'type'})
                && length ($self->{po}{$msgid}{'type'});
         $output .= format_comment($self->{po}{$msgid}{'reference'},": ")
@@ -658,7 +665,8 @@ sub gettextize {
                          'flags'     => $flags,
                          'type'      => $typeorig,
                          'reference' => $reforig,
-                         'conflict'  => 1)
+                         'conflict'  => 1,
+                         'transref'  => $potrans->{po}{$trans}{'reference'})
             unless (defined($pores->{po}{$orig})
                     and ($pores->{po}{$orig}{'msgstr'} eq $trans))
         # FIXME: maybe we should be smarter about what reference should be
@@ -1164,10 +1172,10 @@ sub push {
 sub push_raw {
     my $self=shift;
     my %entry=@_;
-    my ($msgid,$msgstr,$reference,$comment,$automatic,$flags,$type)=
+    my ($msgid,$msgstr,$reference,$comment,$automatic,$flags,$type,$transref)=
         ($entry{'msgid'},$entry{'msgstr'},
          $entry{'reference'},$entry{'comment'},$entry{'automatic'},
-         $entry{'flags'},$entry{'type'});
+         $entry{'flags'},$entry{'type'},$entry{'transref'});
     my $keep_conflict = $entry{'conflict'};
 
 #    print STDERR "Push_raw\n";
@@ -1217,12 +1225,21 @@ sub push_raw {
                  quote_text($msgstr));
 
             if ($keep_conflict) {
-                $msgstr = "#-#-#-#-#  choice  #-#-#-#-#\\n".
-                          $self->{po}{$msgid}{'msgstr'}."\\n".
-                          "#-#-#-#-#  choice  #-#-#-#-#\\n".
-                          "$msgstr";
-                $msgstr = "#-#-#-#-#  choice  #-#-#-#-#\\n".$msgstr
-                    unless ($msgstr =~ m/^#-#-#-#-#  choice  #-#-#-#-#\\n/s);
+                if ($self->{po}{$msgid}{'msgstr'} =~ m/^#-#-#-#-#  .*  #-#-#-#-#\\n/s) {
+                    $msgstr = $self->{po}{$msgid}{'msgstr'}.
+                              "\\n#-#-#-#-#  $transref  #-#-#-#-#\\n".
+                              $msgstr;
+                } else {
+                    $msgstr = "#-#-#-#-#  ".
+                              $self->{po}{$msgid}{'transref'}.
+                              "  #-#-#-#-#\\n".
+                              $self->{po}{$msgid}{'msgstr'}."\\n".
+                              "#-#-#-#-#  $transref  #-#-#-#-#\\n".
+                              $msgstr;
+                }
+                # Every msgid will have the same list of references.
+                # Only keep the last list.
+                $self->{po}{$msgid}{'reference'} = "";
             } else {
             warn wrap_msg(dgettext("po4a",
                                    "Translations don't match for:\n".
@@ -1236,6 +1253,9 @@ sub push_raw {
             }
         }
     }
+    if (defined $transref) {
+        $self->{po}{$msgid}{'transref'} = $transref;
+    }
     if (defined $reference) {
         if (defined $self->{po}{$msgid}{'reference'}) {
             $self->{po}{$msgid}{'reference'} .= " ".$reference;
@@ -1351,8 +1371,14 @@ set, it will return "CHARSET".
 
 sub get_charset() {
     my $self=shift;
+
     $self->{header} =~ /charset=(.*?)[\s\\]/;
-    return $1;
+
+    if (defined $1) {
+        return $1;
+    } else {
+        return "CHARSET";
+    }
 }
 
 =item set_charset($)
