@@ -1,5 +1,5 @@
 # Locale::Po4a::Pod -- Convert POD data to PO file, for translation.
-# $Id: Pod.pm,v 1.21 2006/06/13 19:57:17 nekral-guest Exp $
+# $Id: Pod.pm,v 1.26 2009-12-27 03:43:52 nekral-guest Exp $
 #
 # This program is free software; you may redistribute it and/or modify it
 # under the terms of GPL (see COPYING file).
@@ -13,7 +13,7 @@
 ############################################################################
 
 use Pod::Parser;
-use Locale::Po4a::TransTractor qw(process new);
+use Locale::Po4a::TransTractor qw(process new get_out_charset);
 
 package Locale::Po4a::Pod;
 
@@ -77,6 +77,11 @@ sub command {
 	$self->pushline("=$command\n\n");
     } elsif ($command eq 'over') {
 	$self->pushline("=$command $paragraph".(length($paragraph)?"":"\n\n"));
+    } elsif ($command eq 'encoding') {
+	my $charset = $paragraph;
+	$charset =~ s/^\s*(.*?)\s*$/$1/s;
+	$self->detected_charset($charset)
+	# The =encoding line will be added by docheader
     } else {
 	$paragraph=$self->translate($paragraph,
 				    $self->input_file().":$line_num",
@@ -145,6 +150,16 @@ sub parse {
 }
 
 sub docheader {
+    my $self=shift;
+    my $encoding = $self->get_out_charset();
+    if (    (defined $encoding)
+        and (length $encoding)
+        and ($encoding ne "ascii")) {
+        $encoding = "\n=encoding $encoding\n";
+    } else {
+        $encoding = "";
+    }
+
     return <<EOT;
 
         *****************************************************
@@ -157,7 +172,7 @@ but store the po file used as source file by po4a-translate.
 
 In fact, consider this as a binary, and the po file as a regular .c file:
 If the po get lost, keeping this translation up-to-date will be harder.
-
+$encoding
 EOT
 }
 1;
@@ -169,6 +184,8 @@ EOT
 1;
 __END__
 
+=encoding UTF-8
+
 =head1 NAME
 
 Locale::Po4a::Pod - Convert POD data from/to PO files
