@@ -21,6 +21,7 @@
 #
 ########################################################################
 
+=encoding UTF-8
 
 =head1 NAME
 
@@ -477,7 +478,7 @@ sub get_leading_command {
         $variant = $2;
         $buffer  = $3;
         # read the arguments (if any)
-        while ($buffer =~ m/^\s*([\[\{])(.*)$/s) {
+        while ($buffer =~ m/^\s*$RE_PRE_COMMENT([\[\{])(.*)$/s) {
             my $type = $1;
             my $arg = "";
             my $count = 1;
@@ -485,7 +486,7 @@ sub get_leading_command {
             # stop reading the buffer when the number of ] (or }) matches the
             # the number of [ (or {).
             while ($count > 0) {
-                if ($buffer =~ m/^(.*?)([\[\]\{\}])(.*)$/s) {
+                if ($buffer =~ m/^(.*?$RE_PRE_COMMENT)([\[\]\{\}])(.*)$/s) {
                     $arg .= $1;
                     $buffer = $3;
                     if ($2 eq $type) {
@@ -556,8 +557,8 @@ sub get_trailing_command {
 
     # While the buffer ends by }, consider it is a mandatory argument
     # and extract this argument.
-    while (   $buffer =~ m/^(.*(\{).*)\}$/s
-           or $buffer =~ m/^(.*(\[).*)\]$/s) {
+    while (   $buffer =~ m/^(.*$RE_PRE_COMMENT(\{).*)$RE_PRE_COMMENT\}$/s
+           or $buffer =~ m/^(.*$RE_PRE_COMMENT(\[).*)$RE_PRE_COMMENT\]$/s) {
         my $arg = "";
         my $count = 1;
         $buffer = $1;
@@ -565,7 +566,7 @@ sub get_trailing_command {
         # stop reading the buffer when the number of } (or ]) matches the
         # the number of { (or [).
         while ($count > 0) {
-            if ($buffer =~ m/^(.*)([\{\}\[\]])(.*)$/s) {
+            if ($buffer =~ m/^(.*$RE_PRE_COMMENT)([\{\}\[\]])(.*)$/s) {
                  $arg = $3.$arg;
                  $buffer = $1;
                  if ($2 eq $type) {
@@ -588,7 +589,7 @@ sub get_trailing_command {
     }
 
     # There should now be a command, maybe followed by an asterisk.
-    if ($buffer =~ m/^(.*)$RE_ESCAPE([[:alnum:]]+)(\*?)\s*$/s
+    if ($buffer =~ m/^(.*$RE_PRE_COMMENT)$RE_ESCAPE([[:alnum:]]+)(\*?)\s*$/s
         && defined $separated_command{$2}) {
         $buffer = $1;
         $command = $2;
@@ -907,8 +908,8 @@ sub read {
 =item read_file
 
 Recursively read a file, appending included files which are not listed in the
-@exclude_include array.  Included files are searched in the directory of the
-input document or in a directory listed in the TEXINPUTS environment variable.
+@exclude_include array.  Included files are searched using the B<kpsewhich>
+command from the Kpathsea library.
 
 Except from the file inclusion part, it is a cut and paste from
 Transtractor's read.
@@ -955,18 +956,9 @@ sub read_file {
             }
             if ($include) {
                 # search the file
-                foreach (($my_dirname,
-                          defined($ENV{"TEXINPUTS"})?
-                              split(/:/, $ENV{"TEXINPUTS"}):
-                              ".")) {
-                    if      (-r "$_/$newfilename") {
-                        $newfilename = "$_/$newfilename";
-                        last;
-                    } elsif (-r "$_/$newfilename.tex") {
-                        $newfilename = "$_/$newfilename.tex";
-                        last;
-                    }
-                }
+                open (KPSEA, "kpsewhich " . $newfilename . " |");
+                $newfilename = <KPSEA>;
+
                 push @entries, read_file($self,
                                          $newfilename);
                 if ($tag eq "include") {
@@ -1014,17 +1006,18 @@ newcommands).
 
 sub parse_definition_file {
     my ($self,$filename,$only_try)=@_;
+    my $filename_org = $filename;
 
-    foreach (($my_dirname,
-              defined($ENV{"TEXINPUTS"})?
-                  split(/:/, $ENV{"TEXINPUTS"}):
-                  ".")) {
-        if ((not defined $_) or (not length $_)) {
-            $_ = ".";
-        }
-        if (-r $_."/".$filename) {
-            $filename = $_."/".$filename;
-            last;
+    open (KPSEA, "kpsewhich " . $filename . " |");
+    $filename = <KPSEA>;
+
+    if (not defined $filename) {
+        warn wrap_mod("po4a::tex",
+            dgettext("po4a", "kpsewhich cannot find %s"), $filename_org);
+        if (defined $only_try && $only_try) {
+            return;
+        } else {
+            exit 1;
         }
     }
 
@@ -1087,9 +1080,9 @@ sub parse_definition_line {
         if ($line =~ m/^\s*((?:\{_?\}|\[_?\])*)\s*$/) {
             register_generic_environment("$env,$1");
         }
-    } elsif ($line =~ /^separator\s+(\w+(?:\[#[0-9]+\]))\s+\"(.*)\"\s*$/) {
+    } elsif ($line =~ /^separator\s+(\w+(?:\[#[0-9]+\])?)\s+\"(.*)\"\s*$/) {
         my $env = $1; # This is not necessarily an environment.
-                      # It can also be smth like 'title{#1}'.
+                      # It can also be smth like 'title[#1]'.
         $env_separators{$env} = $2;
     } elsif ($line =~ /^verbatim\s+environment\s+(\w+)\s+$/) {
         register_verbatim_environment($1);
@@ -1108,15 +1101,15 @@ sub is_closed {
     my $opening = 0;
     # FIXME: { and } should not be counted in verbatim blocks
     # Remove comments
-    $tmp =~ s/($RE_PRE_COMMENT)$RE_COMMENT.*//mg;
-    while ($tmp =~ /^.*?(?<!$RE_ESCAPE)(?:$RE_ESCAPE$RE_ESCAPE)*\{(.*)$/s) {
+    $tmp =~ s/$RE_PRE_COMMENT$RE_COMMENT.*//mg;
+    while ($tmp =~ /^.*?$RE_PRE_COMMENT\{(.*)$/s) {
         $opening += 1;
         $tmp = $1;
     }
     $tmp = $paragraph;
     # Remove comments
-    $tmp =~ s/($RE_PRE_COMMENT)$RE_COMMENT.*//mg;
-    while ($tmp =~ /^.*?(?<!$RE_ESCAPE)(?:$RE_ESCAPE$RE_ESCAPE)*\}(.*)$/s) {
+    $tmp =~ s/$RE_PRE_COMMENT$RE_COMMENT.*//mg;
+    while ($tmp =~ /^.*?$RE_PRE_COMMENT\}(.*)$/s) {
         $closing += 1;
         $tmp = $1;
     }
