@@ -1,6 +1,6 @@
 #!/usr/bin/perl -w
 
-# Copyright (c) 2004, 2005 by Nicolas FRANÇOIS <nicolas.francois@centraliens.net>
+# Copyright (c) 2004-2007 by Nicolas FRANÇOIS <nicolas.francois@centraliens.net>
 #
 # This file is part of po4a.
 #
@@ -21,6 +21,8 @@
 #
 ########################################################################
 
+=encoding UTF-8
+
 =head1 NAME
 
 Locale::Po4a::Texinfo - Convert Texinfo documents and derivates from/to PO files
@@ -54,7 +56,7 @@ L<Locale::Po4a::TeX(3pm)|Locale::Po4a::TeX>.
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2004, 2005 by Nicolas FRANÇOIS <nicolas.francois@centraliens.net>.
+Copyright 2004-2007 by Nicolas FRANÇOIS <nicolas.francois@centraliens.net>.
 
 This program is free software; you may redistribute it and/or modify it
 under the terms of GPL (see COPYING file).
@@ -75,14 +77,12 @@ $VERSION= $Locale::Po4a::TeX::VERSION;
 
 use Locale::Po4a::Common;
 use Locale::Po4a::TeX;
-use subs qw(&untranslated     &translate_joined
+use subs qw(
             &parse_definition_file
             &register_generic_command &is_closed &translate_buffer
             &register_verbatim_environment
             &generic_command
             &in_verbatim);
-*untranslated                  = \&Locale::Po4a::TeX::untranslated;
-*translate_joined              = \&Locale::Po4a::TeX::translate_joined;
 *parse_definition_file         = \&Locale::Po4a::TeX::parse_definition_file;
 *register_generic_command      = \&Locale::Po4a::TeX::register_generic_command;
 *register_verbatim_environment = \&Locale::Po4a::TeX::register_verbatim_environment;
@@ -97,6 +97,7 @@ use vars qw($RE_ESCAPE            $ESCAPE
             %commands             %environments
             %command_categories   %separated
             %env_separators       %debug
+            %translate_buffer_env
             @exclude_include      @comments);
 *RE_ESCAPE             = \$Locale::Po4a::TeX::RE_ESCAPE;
 *ESCAPE                = \$Locale::Po4a::TeX::ESCAPE;
@@ -111,6 +112,7 @@ use vars qw($RE_ESCAPE            $ESCAPE
 *separated             = \%Locale::Po4a::TeX::separated;
 *env_separators        = \%Locale::Po4a::TeX::env_separators;
 *debug                 = \%Locale::Po4a::TeX::debug;
+*translate_buffer_env  = \%Locale::Po4a::TeX::translate_buffer_env;
 *exclude_include       = \@Locale::Po4a::TeX::exclude_include;
 *comments              = \@Locale::Po4a::TeX::comments;
 
@@ -118,20 +120,43 @@ $ESCAPE = "\@";
 $RE_ESCAPE = "\@";
 $RE_VERBATIM = "\@example";
 $RE_COMMENT = "\\\@(?:c|comment)\\b";
+$RE_PRE_COMMENT = "(?<!\@)(?:\@\@)*";
 
 my %break_line = ();
 
-foreach (qw/example smallexample verbatim format smallformat exdent
+# translate_line_command indicate if the arguments to the command handled
+# by line_command() should be translated:
+# undefined: arguments are not translated
+# 0:         there should be no arguments
+# 1:         arguments should be translated
+my %translate_line_command = ();
+
+foreach (qw/example smallexample tex display smalldisplay verbatim format smallformat
             flushleft flushright lisp smalllisp ignore/) {
     register_verbatim_environment($_);
-    register_generic_command("*$_,");
-    $commands{$_} = \&environment_command;
+    $commands{$_} = \&environment_line_command;
+    $translate_line_command{$_} = 0; # There should be no arguments
     $break_line{$_} = 1;
 }
 
+my $docheader_pushed = 0;
+# The header shall not be written before the Texinfo header (which include
+# the \input command that define the texinfo macros)
 sub docheader {
-    return "\@c This file was generated with po4a. Translate the source file.\n".
-           "\@c\n";
+    return "";
+}
+
+sub push_docheader {
+    return if $docheader_pushed;
+    my $self = shift;
+    $self->pushline(<<END);
+\@c ===========================================================================
+\@c
+\@c This file was generated with po4a. Translate the source file.
+\@c
+\@c ===========================================================================
+END
+    $docheader_pushed = 1;
 }
 
 sub parse {
@@ -140,6 +165,7 @@ sub parse {
     my $paragraph = ""; # Buffer where we put the paragraph while building
     my @env = (); # environment stack
     my $t = "";
+    $docheader_pushed = 0;
 
   LINE:
     undef $self->{type};
@@ -167,18 +193,27 @@ sub parse {
             # paragraph.
             $paragraph .= $line."\n";
             if (length($paragraph)) {
-                ($t, @env) = translate_buffer($self,$paragraph,@env);
+                ($t, @env) = translate_buffer($self,$paragraph,undef,@env);
                 $self->pushline($t);
                 $paragraph="";
             }
+        } elsif ($line =~ m/^\\input /) {
+            if (length($paragraph)) {
+                ($t, @env) = translate_buffer($self,$paragraph,undef,@env);
+                $self->pushline($t);
+                $paragraph="";
+            }
+            $self->pushline($line."\n");
+            $self->push_docheader();
         } elsif ($line =~ m/^$RE_COMMENT/) {
+            $self->push_docheader();
             $self->pushline($line."\n");
         } elsif (    $closed
                  and ($line =~ /^@([^ ]*?)(?: +(.*))?$/)
                  and (defined $commands{$1})
                  and ($break_line{$1})) {
             if (length($paragraph)) {
-                ($t, @env) = translate_buffer($self,$paragraph,@env);
+                ($t, @env) = translate_buffer($self,$paragraph,undef,@env);
                 $self->pushline($t);
                 $paragraph="";
             }
@@ -189,7 +224,7 @@ sub parse {
                 $arg =~ s/\s*$//s;
                 @args= (" ", $arg);
             }
-            ($t, @env) = &{$commands{$1}}($self, $1, "", \@args, \@env);
+            ($t, @env) = &{$commands{$1}}($self, $1, "", \@args, \@env, 1);
             $self->pushline($t."\n");
         } else {
             # continue the same paragraph
@@ -202,7 +237,7 @@ sub parse {
     }
 
     if (length($paragraph)) {
-        ($t, @env) = translate_buffer($self,$paragraph,@env);
+        ($t, @env) = translate_buffer($self,$paragraph,undef,@env);
         $self->pushline($t);
         $paragraph="";
     }
@@ -211,44 +246,173 @@ sub parse {
 sub line_command {
     my $self = shift;
     my ($command,$variant,$args,$env) = (shift,shift,shift,shift);
-    print "line_command($command,$variant,@$args,@$env)="
+    my $no_wrap = shift;
+    print "line_command($command,$variant,@$args,@$env,$no_wrap)="
         if ($debug{'commands'});
 
     my $translated = $ESCAPE.$command;
     my $line = $args->[1];
     if (defined $line and length $line) {
-        $translated .= " ".$self->translate($line, $self->{ref},
-                                            $command,
-                                            "wrap" => 0);
+        if (    defined $translate_line_command{$command}
+            and $translate_line_command{$command}) {
+            # $no_wrap could be forced to 1, but it should already be set
+            my ($t,$e) = $self->translate_buffer($line,$no_wrap,@$env,$command);
+            $translated .= " ".$t;
+        } else {
+            $translated .= " ".$line;
+        }
     }
     print "($translated,@$env)\n"
         if ($debug{'commands'});
     return ($translated,@$env);
 }
 
-foreach (qw(c appendix section cindex pindex vindex comment subsection
-            subsubsection refill top item chapter settitle setfilename
-            title author bye sp summarycontents contents item noindent)) {
+sub translate_buffer_menu {
+    my ($self,$buffer,$no_wrap,@env) = (shift,shift,shift,@_);
+    print STDERR "translate_buffer_menu($buffer,$no_wrap,@env)="
+        if ($debug{'translate_buffer'});
+
+    my $translated_buffer = "";
+    my $spaces = "";
+    if ($buffer =~ m/(\s*)$/s) {
+        $spaces = $1;
+    }
+
+
+    while ($buffer =~ m/^(.*?)((?:\n|^)\* )(.*)$/s) {
+        my $sep = $2;
+        $buffer = $3;
+        my($t, @e) = $self->translate_buffer_menuentry($1, $no_wrap,
+                                                       @env, "menuentry");
+        $translated_buffer .= $t.$sep;
+    }
+    my($t, @e) = $self->translate_buffer_menuentry($buffer, $no_wrap,
+                                                   @env, "menuentry");
+    $translated_buffer .= $t;
+
+    $translated_buffer .= $spaces;
+
+    print STDERR "($translated_buffer,@env)\n"
+        if ($debug{'translate_buffer'});
+    return ($translated_buffer,@env);
+}
+$translate_buffer_env{"menu"}       = \&translate_buffer_menu;
+$translate_buffer_env{"detailmenu"} = \&translate_buffer_menu;
+$translate_buffer_env{"direntry"}   = \&translate_buffer_menu;
+
+my $menu_width = 78;
+my $menu_sep_width = 30;
+sub translate_buffer_menuentry {
+    my ($self,$buffer,$no_wrap,@env) = (shift,shift,shift,@_);
+    print STDERR "translate_buffer_menuentry($buffer,$no_wrap,@env)="
+        if ($debug{'translate_buffer'});
+
+    my $translated_buffer = "";
+    
+    if (   $buffer =~ m/^(.*?)(::)\s+(.*)$/s
+        or $buffer =~ m/^(.*?: .*?)(\.)\s+(.*)$/s) {
+        my ($name, $sep, $description) = ($1, $2, $3);
+        my ($t, @e) = $self->translate_buffer($name, $no_wrap, @env);
+        $translated_buffer = $t.$sep."  ";
+        my $l = length($translated_buffer) + 2;
+        if ($l < $menu_sep_width-1) {
+            $translated_buffer .= ' 'x($menu_sep_width-1-$l);
+            $l = $menu_sep_width-1;
+        }
+        ($t, @e) = $self->translate_buffer($description, $no_wrap, @env);
+        $t =~ s/\n//sg;
+        $t = Locale::Po4a::Po::wrap($t, $menu_width-$l-2);
+        my $spaces = ' 'x($l+2);
+        $t =~ s/\n/\n$spaces/sg;
+        $translated_buffer .= $t;
+    } else {
+# FIXME: no-wrap if a line start by a space
+        my ($t, @e) = $self->translate_buffer($buffer, $no_wrap, @env);
+        $translated_buffer = $t;
+    }
+
+    print STDERR "($translated_buffer,@env)\n"
+        if ($debug{'translate_buffer'});
+    return ($translated_buffer,@env);
+}
+
+sub translate_buffer_ignore {
+    my ($self,$buffer,$no_wrap,@env) = (shift,shift,shift,@_);
+    print STDERR "translate_buffer_ignore($buffer,$no_wrap,@env);\n"
+        if ($debug{'translate_buffer'});
+    return ($buffer,@env);
+}
+$translate_buffer_env{"ignore"} = \&translate_buffer_ignore;
+
+foreach (qw(appendix section cindex findex kindex opindex pindex vindex subsection
+            dircategory subtitle include
+            exdent center unnumberedsec
+            heading unnumbered unnumberedsubsec
+            unnumberedsubsubsec appendixsec appendixsubsec
+            appendixsubsubsec majorheading chapheading subheading
+            subsubheading shorttitlepage
+            subsubsection top item itemx chapter settitle
+            title author)) {
+    $commands{$_} = \&line_command;
+    $break_line{$_} = 1;
+    $translate_line_command{$_} = 1;
+}
+foreach (qw(c comment set setfilename setchapternewpage vskip synindex
+            syncodeindex need fonttextsize printindex headings finalout sp
+            defcodeindex defindex definfoenclose)) {
     $commands{$_} = \&line_command;
     $break_line{$_} = 1;
 }
-foreach (qw(defcodeindex defcv defcvx deffn deffnx defindex definfoenclose
-            defivar defivarx defmac defmacx defmethod defmethodx defop
-            defopx defopt defoptx defspec defspecx deftp deftpx deftypecv
-            deftypecvx deftypefn deftypefnx deftypefun deftypefunx
-            deftypeivar deftypeivarx deftypemethod deftypemethodx
-            deftypeop deftypeopx deftypevar deftypevarx deftypevr
-            deftypevrx defun defunx defvar defvarx defvr defvrx)) {
+# definfoenclose: command definition => translate?
+foreach (qw(insertcopying page bye summarycontents shortcontents contents
+            noindent)) {
+    $commands{$_} = \&line_command;
+    $break_line{$_} = 1;
+    $translate_line_command{$_} = 0;
+}
+
+foreach (qw(defcv deffn
+            defivar defmac defmethod defop
+            defopt defspec deftp deftypecv
+            deftypefn deftypefun 
+            deftypeivar deftypemethod
+            deftypeop deftypevar deftypevr
+            defun defvar defvr)) {
     $commands{$_} = \&environment_line_command;
+    $translate_line_command{$_} = 1;
+    $break_line{$_} = 1;
+}
+foreach (qw(defcvx deffnx defivarx defmacx defmethodx defopx defoptx
+            defspecx deftpx deftypecvx deftypefnx deftypefunx deftypeivarx
+            deftypemethodx deftypeopx deftypevarx deftypevrx defunx
+            defvarx defvrx)) {
+    $commands{$_} = \&line_command;
+    $translate_line_command{$_} = 1;
     $break_line{$_} = 1;
 }
 
-register_generic_command("*node,");
+foreach (qw(titlefont w i r b sansserif sc slanted strong t cite email
+            footnote indicateurl emph ref xref pxref inforef kbd key
+            acronym),
+# The following commands could cause problems since their arguments
+# have a semantic and a translator could decide not to translate code but
+# still translate theses short words if they appear in another context.
+         qw(file command dfn dmn option math code samp var)) {
+    register_generic_command("-$_,{_}");
+}
+
+register_generic_command("*anchor,{_}");
+register_generic_command("*refill,");
+
+$translate_line_command{'node'} = 1;
+$no_wrap_environments .= " node";
 $break_line{'node'} = 1;
+# @node     Comments,  Minimum, Conventions, Overview
 $commands{'node'} = sub {
     my $self = shift;
     my ($command,$variant,$args,$env) = (shift,shift,shift,shift);
-    print "node($command,$variant,@$args,@$env)="
+    my $no_wrap = shift;
+    print "node($command,$variant,@$args,@$env,$no_wrap)="
         if ($debug{'commands'});
 
     my $translated = $ESCAPE.$command;
@@ -257,7 +421,7 @@ $commands{'node'} = sub {
         my @pointers = split (/, */, $line);
         my @t;
         foreach (@pointers) {
-           push @t, $self->translate($_, $self->{ref}, $command, "wrap" => 0);
+            push @t, $self->translate($_, $self->{ref}, $command, "wrap" => 0);
         }
         $translated .= " ".join(", ", @t);
     }
@@ -270,11 +434,12 @@ $commands{'node'} = sub {
 sub environment_command {
     my $self = shift;
     my ($command,$variant,$args,$env) = (shift,shift,shift,shift);
-    print "environment_command($command,$variant,@$args,@$env)="
+    my $no_wrap = shift;
+    print "environment_command($command,$variant,@$args,@$env,$no_wrap)="
         if ($debug{'commands'});
     my ($t,@e)=("",());
 
-    ($t, @e) = generic_command($self,$command,$variant,$args,$env);
+    ($t, @e) = generic_command($self,$command,$variant,$args,$env,$no_wrap);
     @e = (@$env, $command);
 
     print "($t,@e)\n"
@@ -285,11 +450,12 @@ sub environment_command {
 sub environment_line_command {
     my $self = shift;
     my ($command,$variant,$args,$env) = (shift,shift,shift,shift);
-    print "environment_command_line($command,$variant,@$args,@$env)="
+    my $no_wrap = shift;
+    print "environment_command_line($command,$variant,@$args,@$env,$no_wrap)="
         if ($debug{'commands'});
     my ($t,@e)=("",());
 
-    ($t, @e) = line_command($self,$command,$variant,$args,$env);
+    ($t, @e) = line_command($self,$command,$variant,$args,$env,$no_wrap);
     @e = (@$env, $command);
 
     print "($t,@e)\n"
@@ -312,18 +478,27 @@ sub environment_line_command {
 #    return ($t,@e);
 #}
 #
-foreach (qw(detailmenu menu titlepage enumerate tex group copying
-            quotation documentdescription display smalldisplay cartouche
+foreach (qw(detailmenu menu titlepage group copying
+            documentdescription cartouche
+            direntry
             ifdocbook ifhtml ifinfo ifplaintext iftex ifxml
             ifnotdocbook ifnothtml ifnotinfo ifnotplaintext ifnottex ifnotxml)) {
-    register_generic_command("*$_,");
-    $commands{$_} = \&environment_command;
+    $commands{$_} = \&environment_line_command;
+    $translate_line_command{$_} = 0;
+    $break_line{$_} = 1;
+}
+foreach (qw(enumerate multitable ifclear ifset)) {
+    $commands{$_} = \&environment_line_command;
+    $break_line{$_} = 1;
+}
+foreach (qw(quotation)) {
+    $commands{$_} = \&environment_line_command;
+    $translate_line_command{$_} = 1;
     $break_line{$_} = 1;
 }
 
-# FIXME: maybe format and menu should just be verbatim environments.
-$env_separators{'menu'} = $env_separators{'detailmenu'} = "(?:(?:^|\n)\\\*|::)";
 $env_separators{'format'} = "(?:(?:^|\n)\\\*|END-INFO-DIR-ENTRY|START-INFO-DIR-ENTRY)";
+$env_separators{'multitable'} = "(?:\@item|\@tab)";
 
 my $end_command=$commands{'end'};
 register_generic_command("*end,  ");
@@ -340,17 +515,12 @@ register_generic_command("*table,  ");
 $commands{'table'} = \&environment_command;
 $break_line{'table'} = 1;
 
-register_generic_command("*setchapternewpage,  ");
-$commands{'setchapternewpage'} = \&line_command;
-$break_line{'setchapternewpage'} = 1;
-
 # TODO: is_closed, use a regexp: \ does not escape the closing brace.
 # TBC on LaTeX.
 # In Texinfo, it appears with the "code" command. Maybe this command should
 # be used as verbatim. (Expressions.texi)
 
-# TODO: @include
-# TODO: special function for the indexes
+# TODO: @include @ignore
 
 # TBC: node Indices
 
