@@ -11,11 +11,11 @@ use warnings;
 
 use subs qw(makespace);
 use vars qw($VERSION @ISA @EXPORT);
-$VERSION="0.32";
+$VERSION="0.38";
 @ISA = qw(DynaLoader);
 @EXPORT = qw(new process translate 
              read write readpo writepo
-             getpoout setpoout);
+             getpoout setpoout get_out_charset);
 
 # Try to use a C extension if present.
 eval("bootstrap Locale::Po4a::TransTractor $VERSION");
@@ -29,9 +29,11 @@ use File::Path; # mkdir before write
 use Encode;
 use Encode::Guess;
 
+=encoding UTF-8
+
 =head1 NAME
 
-Po4a TransTractor - Generic trans(lator ex)tractor.
+Locale::Po4a::TransTractor - Generic trans(lator ex)tractor.
 
 =head1 DESCRIPTION
 
@@ -292,6 +294,8 @@ sub process {
     }
     $self->{TT}{'addendum_charset'}=$params{'addendum_charset'};
 
+    chdir $params{'srcdir'}
+	if (defined $params{'srcdir'});
     foreach my $file (@{$params{'po_in_name'}}) {
 	print STDERR "readpo($file)... " if $self->debug();
 	$self->readpo($file);
@@ -310,18 +314,24 @@ sub process {
 	$self->addendum($file) || die "An addendum failed\n";
 	print STDERR "done.\n" if $self->debug();
     }
+    chdir $params{'destdir'}
+	if (defined $params{'destdir'});
     if (defined $params{'file_out_name'}) {
 	print STDERR "write(".$params{'file_out_name'}.")... " 
 	    if $self->debug();
 	$self->write($params{'file_out_name'});
 	print STDERR "done.\n" if $self->debug();
     }
+    chdir $params{'srcdir'}
+	if (defined $params{'srcdir'});
     if (defined $params{'po_out_name'}) {
 	print STDERR "writepo(".$params{'po_out_name'}.")... "
 	     if $self->debug();
 	$self->writepo($params{'po_out_name'});
 	print STDERR "done.\n" if $self->debug();
     }
+    chdir $params{'calldir'}
+	if (defined $params{'calldir'});
     return $self;
 }
 
@@ -337,16 +347,24 @@ sub new {
     ## initialize the plugin
     # prevent the plugin from croaking on the options intended for Po.pm
     $self->{options}{'porefs'} = '';
+    $self->{options}{'copyright-holder'} = '';
+    $self->{options}{'msgid-bugs-address'} = '';
+    $self->{options}{'package-name'} = '';
+    $self->{options}{'package-version'} = '';
     # let the plugin parse the options and such
     $self->initialize(%options);
 
     ## Create our private data
     my %po_options;
     $po_options{'porefs'} = $self->{options}{'porefs'};
+    $po_options{'copyright-holder'} = $options{'copyright-holder'};
+    $po_options{'msgid-bugs-address'} = $options{'msgid-bugs-address'};
+    $po_options{'package-name'} = $options{'package-name'};
+    $po_options{'package-version'} = $options{'package-version'};
     
     # private data
     $self->{TT}=(); 
-    $self->{TT}{po_in}=Locale::Po4a::Po->new();
+    $self->{TT}{po_in}=Locale::Po4a::Po->new(\%po_options);
     $self->{TT}{po_out}=Locale::Po4a::Po->new(\%po_options);
     # Warning, this is an array of array:
     #  The document is splited on lines, and for each
@@ -503,8 +521,6 @@ sub stats   {
     return $_[0]->{TT}{po_in}->stats_get(); 
 }
 
-=cut
-
 =head2 Manipulating addenda
 
 =over 4
@@ -725,7 +741,10 @@ sub shiftline   {
 		     shift @{$_[0]->{TT}{doc_in}}); 
     return ($line,$ref);
 }
-sub unshiftline {  unshift @{$_[0]->{TT}{doc_in}},($_[1],$_[2]);  }
+sub unshiftline {
+	my $self = shift;
+	unshift @{$self->{TT}{doc_in}},@_;
+}
 
 sub pushline    {  push @{$_[0]->{TT}{doc_out}}, $_[1] if defined $_[1]; }
 sub popline     {  return pop @{$_[0]->{TT}{doc_out}};            }
@@ -894,6 +913,14 @@ sub translate {
 #	Encode::from_to($transstring,$self->{TT}{po_in}->get_charset,
 #	    $self->get_out_charset);
 #    }
+
+    if ($options{'wrap'}||0) {
+        $transstring =~ s/( *)$//s;
+        my $trailing_spaces = $1||"";
+        $transstring =~ s/ *$//gm;
+        $transstring .= $trailing_spaces;
+    }
+
     return $transstring;
 }
 
@@ -943,6 +970,9 @@ sub detected_charset {
         $self->{TT}{'file_in_charset'}=$charset;
         if (defined $charset) {
             $self->{TT}{'file_in_encoder'}=find_encoding($charset);
+        } else {
+            $self->{TT}{ascii_input}=1;
+            $self->{TT}{utf_mode}=0;
         }
     }
 
